/*-------------------------------------------------------------------------*/
/*                                                                         */
/*           !!! This Header shut not be removed !!!                       */
/*                                                                         */
/*  Aci.h - the header file for ACI users                                  */
/*                                                                         */
/*  Version: 3.01 or later                                                 */
/*-------------------------------------------------------------------------*/
/*  Copyright by GRAU Storage Systems GmbH & Co. 1996, 1997                */
/*-------------------------------------------------------------------------*/


#ifndef   ACI_INCL

#define   ACI_INCL    1

#include <sys/types.h>
#ifdef _WIN32
 #include <winrpc.h>
#else
 #include <rpc/rpc.h>    /* for bool_t   */
 #include <netinet/in.h>
#endif
#include "derrno.h"     /* das errnos   */

/*-------------------------------------------------------------------------*/
/*                                                                         */
/*-------------------------------------------------------------------------*/

enum aci_media { ACI_3480 = 1, ACI_OD_THICK, ACI_OD_THIN,
                 ACI_DECDLT, ACI_8MM, ACI_4MM, ACI_D2, ACI_VHS, ACI_3590,
                 ACI_CD, ACI_TRAVAN, ACI_DTF, ACI_BETACAM, ACI_AUDIO_TAPE,
                 ACI_BETACAML, ACI_SONY_AIT};

enum aci_command {ACI_ADD = 1, ACI_MODIFY, ACI_DELETE};

enum aci_drive_status
{
   ACI_DRIVE_DOWN = 1, 
   ACI_DRIVE_UP, 
   ACI_DRIVE_FDOWN, 
   ACI_DRIVE_FUP, 
   ACI_DRIVE_EXUP
};


enum aci_volser_status
{
   ACI_VOLSER_DOWN = 1, ACI_VOLSER_UP
};

/*-------------------------------------------------------------------------*/
/* defines for ACI constants                                               */
/*-------------------------------------------------------------------------*/

#define ACI_COORD_LEN   11      /* should match das.h setting */
#define ACI_VOLSER_LEN  17      /* should match das.h setting */
#define ACI_NAME_LEN    65      /* should match das.h setting */
#define ACI_REQTYPE_LEN 10      /* should match das.h setting */
#define ACI_DRIVE_LEN   10      /* should match das.h setting */
#define ACI_AMU_DRIVE_LEN 3     /* should be das.h setting +1 */
#define ACI_OPTIONS_LEN 3       /* should match das.h setting */
#define ACI_MEDIA_LEN 3         /* should match das.h setting */
#define ACI_MAX_RANGES  10      /* should match das.h setting */
#define ACI_RANGE_LEN   100     /* should match das.h setting */
#define ACI_RANGE_LEN2  200     /* should match das.h setting */
#define ACI_POOLNAME_LEN  17    /* should match das.h setting */
#define ACI_VOLSERSTATUS_LEN  5 /* should match das.h setting */

#define ACI_MAX_REQ_ENTRIES     15    /* should match das.h setting */
#define ACI_MAX_DRIVE_ENTRIES   15 /* should match das.h setting */
#define ACI_MAX_DRIVE_ENTRIES2 250 /* should match das.h setting */

#define ACI_MAX_VOLSER_ENTRIES  50 /* should match das.h setting */

#define ACI_MAX_VERSION_LEN 20   /*           including '\0'   */
#define ACI_MAX_QUERY_VOLSRANGE 1000

#define ACI_EI_MAX_RANGE 800

#define ACI_VOLSER_ATTRIB_MOUNTED   'M'
#define ACI_VOLSER_ATTRIB_EJECTED   'E'
#define ACI_VOLSER_ATTRIB_OCCUPIED  'O'
#define ACI_VOLSER_ATTRIB_UNDEFINED 'U'
#define ACI_VOLSER_ATTRIB_EMPTY     'Y'
#define ACI_VOLSER_ATTRIB_REVERSSIDEMOUNTED 'R'
#define ACI_VOLSER_ATTRIB_IN_JUKEBOX 'J'
#define ACI_VOLSER_ATTRIB_INITIAL   'I'
#define ACI_VOLSER_ATTRIB_TEMP_HERE 'T'
#define ACI_VOLSER_ATTRIB_TEMP_AWAY 'A'


/*-------------------------------------------------------------------------*/
/*                                                                         */
/*-------------------------------------------------------------------------*/

struct aci_vol_desc {
        char coord[ACI_COORD_LEN];
        char owner;
        char attrib;
        char type;
        char volser[ACI_VOLSER_LEN];
        char vol_owner;
        int use_count;
        int crash_count;
};


struct aci_drive_entry {
    char drive_name[ACI_DRIVE_LEN];
    char amu_drive_name[ACI_AMU_DRIVE_LEN];
    enum aci_drive_status drive_state;
    char type;
    char system_id[ACI_NAME_LEN];
    char clientname[ACI_NAME_LEN];
    char volser[ACI_VOLSER_LEN];
    bool_t cleaning;
    short clean_count;
};


/********listd3*******************************/

struct aci_ext_drive_entry {
    char drive_name[ACI_DRIVE_LEN];
    char amu_drive_name[ACI_AMU_DRIVE_LEN];
    enum aci_drive_status drive_state;
    char type;
    char system_id[ACI_NAME_LEN];
    char clientname[ACI_NAME_LEN];
    char volser[ACI_VOLSER_LEN];
    bool_t cleaning;
    short clean_count;
    int  mount;
    int  keep;
};

/********************************************/


struct aci_volser_entry {
    char ClientName[ACI_NAME_LEN];
    char VolserRange[ACI_RANGE_LEN];
    enum aci_volser_status volser_state;
};

struct aci_voltodrive_entry {
    char Drive[ACI_DRIVE_LEN];
    char VolserRange[ACI_RANGE_LEN];
};

struct aci_client_entry {
        char clientname[ACI_NAME_LEN];
        struct in_addr ip_addr;
        bool_t avc;
        bool_t complete_access;
        bool_t dismount;
        char volser_range[ACI_MAX_RANGES][ACI_RANGE_LEN];
        char drive_range [ACI_RANGE_LEN];
};


struct aci_req_entry {
        u_long request_no;
        u_long individ_no;
        char clientname [ACI_NAME_LEN];
        char req_type [ACI_REQTYPE_LEN + 1];
};

/*-------------------------------------------------------------------------*/
/*                                                                         */
/*-------------------------------------------------------------------------*/

struct aci_volserinfo
{
       char            volser [ ACI_VOLSER_LEN ];
       enum aci_media  media_type;
       char            attrib;
};


struct aci_ei_info
{
       char            volser [ ACI_VOLSER_LEN ];
       char            media_type [ACI_DRIVE_LEN];
       int             errcode;
};


/*-------------------------------------------------------------------------*/
/*  Doubleside medium                          17.6.96 (ak)                */
/*-------------------------------------------------------------------------*/
struct aci_sideinfo
{
       char            cVolumeSide;
       char            szVolser[ACI_VOLSER_LEN];
};


/*-------------------------------------------------------------------------*/
/*                                                                         */
/*-------------------------------------------------------------------------*/

#if __STDC__ == 1
extern int aci_cancel (u_long);

extern int clientaccess (char *, enum aci_command, char *, enum aci_media, char *);
extern int clientstatus (char *, struct aci_client_entry *);

extern int aci_dismount (char *, enum aci_media);
extern int aci_driveaccess  (char *, char *, enum aci_drive_status);
extern int aci_volseraccess  (char *, char *, enum aci_volser_status);
extern int aci_drivestatus  (char *, struct aci_drive_entry *[]);
extern int aci_volserstatus  (char *, int * ,struct aci_volser_entry *);
extern int aci_drivestatus2 (char *, struct aci_drive_entry *[]);
extern int aci_drivestatus3 (char *, struct aci_ext_drive_entry *[]);
extern int aci_eject (char *, char *, enum aci_media);
extern int aci_eject2 (char *, char *, enum aci_media ,int *, struct aci_ei_info *);
extern int aci_eject_complete( char *, char *, enum aci_media );
extern int aci_eject2_complete( char *, char *, enum aci_media, int *, struct aci_ei_info *);
extern int aci_force (char *);
extern int aci_foreign (enum aci_command, char *, enum aci_media, char *,short);
extern int aci_init (void);
extern int aci_insert (char *, char *[], enum aci_media *);
extern int aci_list (char *, struct aci_req_entry *[]);
extern int aci_mount (char *, enum aci_media, char *);
extern int aci_move (char *, enum aci_media, char *);
extern int aci_carry( char *, char *, char *, int *);
extern void aci_perror (char *);
extern int aci_register (char *, char *, enum aci_command,bool_t,bool_t,bool_t);
extern int aci_shutdown (char *);
extern int aci_view (char *, enum aci_media, struct aci_vol_desc *);
extern int aci_insert2 ( char *, char *, char *, int *, struct aci_ei_info * );
extern int aci_ejectclean( char *, char *, int *, struct aci_ei_info * );

extern int aci_initialize( void );
extern int aci_qversion( char *, char * );
extern int aci_qvolsrange( char *, char *, int, char *, int *,
                           struct aci_volserinfo *               );
extern int aci_inventory( void );
extern int aci_partial_inventory( char *, char *);
extern int aci_scratch_set (char *, enum aci_media , char * );
extern int aci_scratch_get (char *, enum aci_media , char * );
extern int aci_scratch_unset (char *, enum aci_media , char * );
extern int aci_scratch_info (char *,  enum aci_media , long *, long *);

extern int aci_flip( char * );
extern int aci_getvolsertoside( char *, struct aci_sideinfo * );
extern int aci_getvolsertodrive(char *,int *, struct aci_voltodrive_entry * );

extern int aci_robhome( char * );
extern int aci_robstat( char *, char *  );
extern int aci_unload( char * );

#else
#ifdef _WIN32
extern int __cdecl aci_barcode(char *cRobNum,char *Action);
extern int __cdecl aci_cancel(unsigned long request_no);
extern int __cdecl aci_carry(char *pszVolser,char *pszSourceCoord,char *pszTargetCoord,int *iAmuErr);
extern int __cdecl aci_cleandrive(char *drive);
extern int __cdecl aci_clientaccess(char *clientname,int action,char *volser_range,int type,char *drive_range);
extern int __cdecl aci_clientstatus(char *clientname,struct aci_client_entry *client_status);
extern int __cdecl aci_dismount(char *volser,int type);
extern int __cdecl aci_driveaccess(char *clientname,char *drive,int drive_state);
extern int __cdecl aci_drivestatus(char *clientname,struct aci_drive_entry **drive_state );
extern int __cdecl aci_drivestatus2(char *clientname,struct aci_drive_entry **drive_state );
extern int __cdecl aci_drivestatus3(char *clientname,struct aci_ext_drive_entry **drive_state );
extern int __cdecl aci_dismount(char *volser,int type);
extern int __cdecl aci_driveaccess(char *clientname,char *drive,int drive_state);
extern int __cdecl aci_drivestatus(char *clientname,struct aci_drive_entry **drive_state );
extern int __cdecl aci_drivestatus2(char *clientname,struct aci_drive_entry **drive_state );
extern int __cdecl aci_drivestatus3(char *clientname,struct aci_ext_drive_entry **drive_state );
extern int __cdecl aci_eject(char *areaname,char *volser_range,int type);
extern int __cdecl aci_eject2(char *areaname,char *volser_range,int type,int *pnActualCount,struct _aci_ei_info *pstEiInfo);
extern int __cdecl aci_ejectclean(char *areaname,char *CleanPoolname,int *pnActualCount,struct _aci_ei_info *pstEiInfo);
extern int __cdecl aci_eject_complete(char *pszAreaName,char *pszVolserRange,int eMediaType);
extern int __cdecl aci_eject2_complete(char *pszAreaName,char *pszVolserRange,int eMediaType,int *pnActualCount,struct _aci_ei_info *pstEiInfo);
extern int __cdecl aci_flip(char *pszDrive);
extern int __cdecl aci_force(char *drive);
extern int __cdecl aci_foreign(int action,char *volser,int type,char *coord,short position);
extern int __cdecl aci_init(void );
extern int __cdecl aci_initialize(void );
extern int __cdecl aci_insert(char *areaname,char **vol_ranges ,int *type);
extern int __cdecl aci_insert2(char *insertopt,char *areaname,char *CleanPoolname,int *pnActualCount,struct _aci_ei_info *pstEiInfo);
extern int __cdecl aci_inventory(void );
extern int __cdecl aci_killamu(void );
extern int __cdecl aci_list(char *clientname,struct aci_req_entry **req_status );
extern int __cdecl aci_mount(char *volser,int type,char *drive);
extern int __cdecl aci_move(char *volser,int type,char *dynarea);
extern void __cdecl aci_perror(char *aci_text);
extern int __cdecl aci_partial_inventory(char *pszSourceCoord,char *pszTargetCoord);
extern int __cdecl aci_qversion(char *pszACIVersion,char *pszDASVersion);
extern int __cdecl aci_qvolsrange(char *pszBeginVolser,char *pszEndVolser,int nCount,char *pszClientName,int *pnActualCount,struct aci_volserinfo *pstVolserInfo);
extern int __cdecl aci_register(char *clientname,char *hostname,int action,unsigned char avc,unsigned char complete_access,unsigned char dismount);
extern int __cdecl aci_robhome(char *pszRob);
extern int __cdecl aci_robstat(char *pszRobot,char *pszAction);
extern int __cdecl aci_scratch_set(char *pszPoolname,int eMediaType,char *pszVolser);
extern int __cdecl aci_scratch_get(char *pszPoolname,int eMediaType,char *pszVolser);
extern int __cdecl aci_scratch_unset(char *pszPoolname,int eMediaType,char *pszVolser);
extern int __cdecl aci_scratch_info(char *pszPoolname,int eMediaType,long *plVolserCount,long *plScratchCount);
extern int __cdecl aci_shutdown(char *now);
extern int __cdecl aci_switch(char *Switch);
extern int __cdecl aci_turn(char *pszDrive);
extern int __cdecl aci_unload(char *pszDrive);
extern int __cdecl aci_view(char *volser,int type,struct aci_vol_desc *vol_desc);
extern int __cdecl aci_volseraccess(char *clientname,char *VolRange,int volser_state);
extern int __cdecl aci_volserstatus(char *clientname,int *num,struct aci_volser_entry *volser_state);
extern int __cdecl aci_getvolsertoside(char *pszVolser,struct aci_sideinfo *pstSI);
extern int __cdecl aci_getvolsertodrive(char *Drive,int *num,struct aci_voltodrive_entry *drive_info);
#else
extern int aci_cancel ();
extern int clientaccess ();
extern int clientstatus ();
extern int aci_dismount ();
extern int aci_flip ();
extern int aci_driveaccess  ();
extern int aci_volseraccess  ();
extern int aci_drivestatus  ();
extern int aci_drivestatus2 ();
extern int aci_drivestatus3 ();
extern int aci_volserstatus ();
extern int aci_eject ();
extern int aci_eject2 ();
extern int aci_eject_complete ();
extern int aci_eject2_complete ();
extern int aci_force ();
extern int aci_foreign ();
extern int aci_init ();
extern int aci_insert ();
extern int aci_list ();
extern int aci_mount ();
extern int aci_move ();
extern int aci_carry ();
extern void aci_perror ();
extern int aci_register ();
extern int aci_shutdown ();
extern int aci_view ();
extern int aci_insertgen ();
extern int aci_ejectclean ();

extern int aci_initialize();
extern int aci_qversion( );
extern int aci_qvolsrange( );
extern int aci_inventory( );
extern int aci_partial_inventory( );
extern int aci_scratch_set ( );
extern int aci_scratch_get ( );
extern int aci_scratch_unset ();
extern int aci_scratch_info ( );

extern int aci_getvolsertoside( );
extern int aci_getvolsertodrive( );

extern int aci_unload ( );
extern int aci_robstat ( );
extern int aci_robhome ( );

#endif

#endif


/*-------------------------------------------------------------------------*/
/*                                                                         */
/*-------------------------------------------------------------------------*/
/*  global object for error return from aci_ call */
#ifdef _WIN32

__declspec (dllimport) extern  int  d_errno;     
__declspec (dllimport) extern  char szRetDrive[15];
__declspec (dllimport) extern  char  szVolser[17];
__declspec (dllimport) extern  char  szSourceCoord[11];
__declspec (dllimport) extern  char  szTargetCoord[11];

__declspec (dllimport) extern char *pszAciErrTab[];
__declspec (dllimport) extern char *pszDasErrTab[];
__declspec (dllimport) extern int  nDasErrTabCount;

#else
extern int  d_errno;     /*  global object for error return from aci_ call */

extern char *pszAciErrTab[];
extern char *pszDasErrTab[];
extern int  nDasErrTabCount;

extern  char  szRetDrive[15];
extern  char  szVolser[17];
extern  char  szSourceCoord[11];
extern  char  szTargetCoord[11];

#endif


#endif                  /* ACI_INCL   */


/*-------------------------------------------------------------------------*/
/*  E n d                                                                  */
/*-------------------------------------------------------------------------*/


