/* $Id: winrpc.ht,v 1.17 1997/03/19 21:09:38 al Exp $ */
/*
**      WINRPC.H -- ONC RPC for Win32
**
**  This header file is the companion to WINRPC32.DLL.

**
**  WINRPC.H describes many of the internal data structures and functions
**  of the internals of ONC RPC.  Some of this may be useful for supporting
**  hand-coded ONC RPC applications ported to the Windows environment.
**  The preferred method is to automatically generate the RPC programming
**  interface code using the EZRPC compiler provided in NobleNet's WINRPC
**  toolkit.  WINRPC is a trademark of NobleNet, Inc.
**
**  Copyright (C) 1991-1997, NobleNet, Inc.
*/

 /*
 * Sun RPC is a product of Sun Microsystems, Inc. and is provided for
 * unrestricted use provided that this legend is included on all tape
 * media and as a part of the software program in whole or part.  Users
 * may copy or modify Sun RPC without charge, but are not authorized
 * to license or distribute it to anyone else except as part of a product or
 * program developed by the user.
 * 
 * SUN RPC IS PROVIDED AS IS WITH NO WARRANTIES OF ANY KIND INCLUDING THE
 * WARRANTIES OF DESIGN, MERCHANTIBILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE, OR ARISING FROM A COURSE OF DEALING, USAGE OR TRADE PRACTICE.
 * 
 * Sun RPC is provided with no support and without any obligation on the
 * part of Sun Microsystems, Inc. to assist in its use, correction,
 * modification or enhancement.
 * 
 * SUN MICROSYSTEMS, INC. SHALL HAVE NO LIABILITY WITH RESPECT TO THE
 * INFRINGEMENT OF COPYRIGHTS, TRADE SECRETS OR ANY PATENTS BY SUN RPC
 * OR ANY PART THEREOF.
 * 
 * In no event will Sun Microsystems, Inc. be liable for any lost revenue
 * or profits or other special, indirect and consequential damages, even if
 * Sun has been advised of the possibility of such damages.
 * 
 * Sun Microsystems, Inc.
 * 2550 Garcia Avenue
 * Mountain View, California  94043
 */

#ifndef _WINRPC_H_
#define _WINRPC_H_


#ifdef USEXDRSTDIO
#include <stdio.h>      /* for the FILE definition in xdrstdio */
#endif
#include <memory.h>
#include <stdlib.h>
#include <string.h>

#include <io.h>
#include <windows.h>
#include <winsock.h>

#ifndef const__
#define const__ const
#endif /* const__ */

/* winrpc32.dll for nt/95 is built with _cdecl. */

#define WINRPCAPI    




#define RPC_ANYSOCK ((SOCKET) (~0))

typedef int         bool_t;
typedef int         enum_t;
typedef char FAR *  caddr_t;

#define MAX_AUTH_BYTES  400
#define MAXNETNAMELEN   255 /* maximum length of network user's name */

#ifndef FALSE
#define FALSE   (0)
#define TRUE    (1)
#endif

#define __dontcare__    -1
#ifndef NULL
#define NULL 0
#endif

#ifndef        MAXHOSTNAMELEN
#define        MAXHOSTNAMELEN  64
#endif

enum xdr_op {
	XDR_ENCODE=0,
	XDR_DECODE=1,
	XDR_FREE=2
};

#define BYTES_PER_XDR_UNIT  (4)
#define RNDUP(x)  \
	((((x) + BYTES_PER_XDR_UNIT - 1) / BYTES_PER_XDR_UNIT) * BYTES_PER_XDR_UNIT)


typedef struct xdr {
	enum xdr_op x_op;       /* operation; fast additional param */
	struct xdr_ops {
		bool_t  (WINRPCAPI *x_getlong)(struct xdr FAR *, long FAR *);
				/* get a long from underlying stream */
		bool_t  (WINRPCAPI *x_putlong)(struct xdr FAR *, long FAR *);  
				/* put a long to " */
		bool_t  (WINRPCAPI *x_getbytes)(struct xdr FAR *, caddr_t, u_int);
				/* get some bytes from " */
		bool_t  (WINRPCAPI *x_putbytes)(struct xdr FAR *, caddr_t, u_int);
				/* put some bytes to " */
		u_int   (WINRPCAPI *x_getpostn)(struct xdr FAR *);
				/* returns bytes off from beginning */
		bool_t  (WINRPCAPI *x_setpostn)(struct xdr FAR *, u_int);
				/* lets you reposition the stream */
		long FAR *(WINRPCAPI *x_inline)(struct xdr FAR *, u_int);  
				/* buf quick ptr to buffered data */
		void    (WINRPCAPI *x_destroy)(struct xdr FAR *);  
				/* free privates of this xdr_stream */
	} FAR *x_ops;
	caddr_t     x_public;   /* users' data */
	caddr_t     x_private;  /* pointer to private data */
	caddr_t     x_base;     /* private used for position info */
	int     x_handy;    /* extra private word */
} XDR;
typedef bool_t (WINRPCAPI *xdrproc_t)(XDR FAR *, void FAR *);


struct bcast_rply{
	u_int    errorcode;
	struct sockaddr_in FAR * from;
	void               FAR * answer;
	xdrproc_t                  DestroyProc;
	xdrproc_t                  XdrFreeProc;
};
typedef struct bcast_rply bCast;


/*
 * Operations defined on a XDR handle
 *
 * XDR      *xdrs;
 * long     *longp;
 * caddr_t   addr;
 * u_int     len;
 * u_int     pos;
 */
#define XDR_GETLONG(xdrs, longp)            \
	(*(xdrs)->x_ops->x_getlong)((XDR FAR *)xdrs, (long FAR *)longp)
#define xdr_getlong(xdrs, longp)            \
	(*(xdrs)->x_ops->x_getlong)((XDR FAR *)xdrs, (long FAR *)longp)

#define XDR_PUTLONG(xdrs, longp)            \
	(*(xdrs)->x_ops->x_putlong)((XDR FAR *)xdrs, (long FAR *)longp)
#define xdr_putlong(xdrs, longp)            \
	(*(xdrs)->x_ops->x_putlong)((XDR FAR *)xdrs, (long FAR *)longp)

#define XDR_GETBYTES(xdrs, addr, len)           \
	(*(xdrs)->x_ops->x_getbytes)((XDR FAR *)xdrs, (caddr_t)addr, len)
#define xdr_getbytes(xdrs, addr, len)           \
	(*(xdrs)->x_ops->x_getbytes)((XDR FAR *)xdrs, (caddr_t)addr, len)

#define XDR_PUTBYTES(xdrs, addr, len)           \
	(*(xdrs)->x_ops->x_putbytes)((XDR FAR *)xdrs, (caddr_t)addr, len)
#define xdr_putbytes(xdrs, addr, len)           \
	(*(xdrs)->x_ops->x_putbytes)((XDR FAR *)xdrs, (caddr_t)addr, len)

#define XDR_GETPOS(xdrs)                \
	(*(xdrs)->x_ops->x_getpostn)((XDR FAR *)xdrs)
#define xdr_getpos(xdrs)                \
	(*(xdrs)->x_ops->x_getpostn)((XDR FAR *)xdrs)

#define XDR_SETPOS(xdrs, pos)               \
	(*(xdrs)->x_ops->x_setpostn)((XDR FAR *)xdrs, pos)
#define xdr_setpos(xdrs, pos)               \
	(*(xdrs)->x_ops->x_setpostn)((XDR FAR *)xdrs, pos)

#define XDR_INLINE(xdrs, len)               \
	(*(xdrs)->x_ops->x_inline)((XDR FAR *)xdrs, len)
#define xdr_inline(xdrs, len)               \
	(*(xdrs)->x_ops->x_inline)((XDR FAR *)xdrs, len)

#define XDR_DESTROY(xdrs)               \
	if ((xdrs)->x_ops->x_destroy)           \
		(*(xdrs)->x_ops->x_destroy)((XDR FAR *)xdrs)
#define xdr_destroy(xdrs)               \
	if ((xdrs)->x_ops->x_destroy)           \
		(*(xdrs)->x_ops->x_destroy)((XDR FAR *)xdrs)

/*
 * Support struct for discriminated unions.
 * You create an array of xdrdiscrim structures, terminated with
 * a entry with a null procedure pointer.  The xdr_union routine gets
 * the discriminant value and then searches the array of structures
 * for a matching value.  If a match is found the associated xdr routine
 * is called to handle that part of the union.  If there is
 * no match, then a default routine may be called.
 * If there is no match and no default routine it is an error.
 */
#define NULL_xdrproc_t ((xdrproc_t)0)
struct xdr_discrim {
	int value;
	xdrproc_t proc;
};

/*
 * In-line routines for fast encode/decode of primitve data types.
 * Caveat emptor: these use single memory cycles to get the
 * data from the underlying buffer, and will fail to operate
 * properly if the data is not aligned.  The standard way to use these
 * is to say:
 *  if ((buf = XDR_INLINE(xdrs, count)) == NULL)
 *      return (FALSE);
 *  <<< macro calls >>>
 * where ``count'' is the number of bytes of data occupied
 * by the primitive data types.
 *
 * N.B. and frozen for all time: each data type here uses 4 bytes
 * of external representation.
 */
#define IXDR_GET_LONG(buf)      ((long)ntohl((u_long)*(buf)++))
#define IXDR_PUT_LONG(buf, v)       (*(buf)++ = (u_long)htonl((u_long)v))

#define IXDR_GET_BOOL(buf)      ((bool_t)IXDR_GET_LONG(buf))
#define IXDR_GET_ENUM(buf, t)       ((t)IXDR_GET_LONG(buf))
#define IXDR_GET_U_LONG(buf)        ((u_long)IXDR_GET_LONG(buf))
#define IXDR_GET_SHORT(buf)     ((short)IXDR_GET_LONG(buf))
#define IXDR_GET_U_SHORT(buf)       ((u_short)IXDR_GET_LONG(buf))

#define IXDR_PUT_BOOL(buf, v)       IXDR_PUT_LONG((buf), ((long)(v)))
#define IXDR_PUT_ENUM(buf, v)       IXDR_PUT_LONG((buf), ((long)(v)))
#define IXDR_PUT_U_LONG(buf, v)     IXDR_PUT_LONG((buf), ((long)(v)))
#define IXDR_PUT_SHORT(buf, v)      IXDR_PUT_LONG((buf), ((long)(v)))
#define IXDR_PUT_U_SHORT(buf, v)    IXDR_PUT_LONG((buf), ((long)(v)))


/*
 * Common opaque bytes objects used by many rpc protocols;
 * declared here due to commonality.
 */
#define MAX_NETOBJ_SZ 1024 
struct netobj {
	u_int   n_len;
	char    FAR *n_bytes;
};
typedef struct netobj netobj;

/*
 * Status returned from authentication check
 */

typedef u_long u_int32; /* 32-bit unsigned integers */

union des_block {
	struct {
		u_int32 high;
		u_int32 low;
	} key;
	char c[8];
};
typedef union des_block des_block;


 bool_t WINRPCAPI xdr_des_block(XDR FAR *xdrs, des_block FAR *blkp);


/*
 * Authentication info.  Opaque to client.
 */
struct opaque_auth {
	enum_t  oa_flavor;      /* flavor of auth */
	caddr_t oa_base;        /* address of more auth stuff */
	u_int   oa_length;      /* not to exceed MAX_AUTH_BYTES */
};


/*
 * Auth handle, interface to client side authenticators.
 */

typedef struct auth {
	struct  opaque_auth ah_cred;
	struct  opaque_auth ah_verf;
	union   des_block   ah_key;
	struct auth_ops {
		void    (WINRPCAPI *ah_nextverf)(struct auth FAR *);
		int (WINRPCAPI *ah_marshal)(struct auth FAR *, XDR FAR*);
				/* nextverf & serialize */
		int (WINRPCAPI *ah_validate)(struct auth FAR*,
						  struct opaque_auth FAR*);
				/* validate verifier */
		int (WINRPCAPI *ah_refresh)(struct auth FAR*);
				/* refresh credentials */
		void    (WINRPCAPI *ah_destroy)(struct auth FAR*);
				/* destroy this structure */
	} FAR *ah_ops;
	caddr_t ah_private;
} AUTH;

/*
 * Authentication ops.
 * The ops and the auth handle provide the interface to the authenticators.
 *
 * AUTH *auth;
 * XDR  *xdrs;
 * struct opaque_auth verf;
 */

#define AUTH_NEXTVERF(auth)     \
		((*((auth)->ah_ops->ah_nextverf))( (auth) ))
#define auth_nextverf(auth)     \
		((*((auth)->ah_ops->ah_nextverf))( (auth) ))

#define AUTH_MARSHALL(auth, xdrs)   \
		((*((auth)->ah_ops->ah_marshal))( (auth), \
		xdrs))
#define auth_marshall(auth, xdrs)   \
		((*((auth)->ah_ops->ah_marshal))( (auth), \
		xdrs))

#define AUTH_VALIDATE(auth, verfp)  \
		((*((auth)->ah_ops->ah_validate))( (auth), \
		verfp))
#define auth_validate(auth, verfp)  \
		((*((auth)->ah_ops->ah_validate))( (auth), \
		verfp))
#define AUTH_REFRESH(auth)      \
		((*((auth)->ah_ops->ah_refresh))( (auth) ))
#define auth_refresh(auth)      \
		((*((auth)->ah_ops->ah_refresh))( (auth) ))

#define AUTH_DESTROY(auth)      \
		((*((auth)->ah_ops->ah_destroy))( (auth) ))
#define auth_destroy(auth)      \
		((*((auth)->ah_ops->ah_destroy))( (auth) ))

/*
 * These are the various implementations of client side authenticators.
 */

/*
 * Unix style authentication
 * AUTH *authunix_create(machname, uid, gid, len, aup_gids)
 *  char *machname;
 *  u_long uid;
 *  u_long gid;
 *  int len;
 *  u_long *aup_gids;
 */



#define AUTH_NONE   0       /* no authentication */
#define AUTH_NULL   0       /* backward compatibility */
#define AUTH_UNIX   1       /* unix style (uid, gids) */
#define AUTH_SHORT  2       /* short hand unix style */
#define AUTH_DES    3       /* des style (encrypted timestamps) */

#define MAX_MARSHEL_SIZE 20



/*
 * The system is very weak.  The client uses no encryption for  it
 * credentials and only sends null verifiers.  The server sends backs
 * null verifiers or optionally a verifier that suggests a new short hand
 * for the credentials.
 */


/* The machine name is part of a credential; it may not exceed 255 bytes */
#define MAX_MACHINE_NAME 255

/* gids compose part of a credential; there may not be more than 16 of them */
#define NGRPS 16

/*
 * Unix style credentials.
 */
typedef u_long gid_t; 

struct authunix_parms {
	u_long    aup_time;
	char FAR *aup_machname;
	u_long      aup_uid;
	u_long      aup_gid;
	u_int     aup_len;
	gid_t FAR *aup_gids;
};

/* 
 * If a response verifier has flavor AUTH_SHORT, 
 * then the body of the response verifier encapsulates the following structure;
 * again it is serialized in the obvious fashion.
 */
struct short_hand_verf {
	struct opaque_auth new_cred;
};

enum clnt_stat {
	RPC_UNUSEDERROR = 999	/* see RPC_ error code #defines below */
};

/*
 * Client rpc handle.
 * Created by individual implementations, see e.g. rpc_udp.c.
 * Client is responsible for initializing auth, see e.g. auth_none.c.
 */
struct rpc_err;		
typedef struct CLIENT {
	AUTH    FAR * cl_auth;           /* authenticator */
	struct clnt_ops {
		enum clnt_stat  (WINRPCAPI *cl_call)(struct CLIENT FAR *,
												u_long,
												xdrproc_t,
												char FAR *,
												xdrproc_t,
												char FAR *,
												struct timeval);
							/* call remote procedure */
		void    (WINRPCAPI *cl_abort)(struct CLIENT FAR *);
							/* abort a call */
		void    (WINRPCAPI *cl_geterr)(struct CLIENT FAR *, struct rpc_err FAR
		*);
							/* get specific error code */
		bool_t  (WINRPCAPI *cl_freeres)(struct CLIENT FAR *, xdrproc_t, caddr_t);
							/* frees results */
		void    (WINRPCAPI *cl_destroy)(struct CLIENT FAR *);
							/* destroy this structure */
		bool_t  (WINRPCAPI *cl_control)(struct CLIENT FAR *, int, char FAR *);
							/* the ioctl() of rpc */
	} FAR *cl_ops;
	caddr_t	cl_private; /* private stuff */
} CLIENT;


enum xprt_stat {
	XPRT_DIED,
	XPRT_MOREREQS,
	XPRT_IDLE
};



/*
 * Server side transport handle
 */

struct rpc_msg;	

typedef struct  SVCXPRT{
	SOCKET  xp_sock;
	u_short     xp_port;     /* associated port number */
	struct xp_ops {
		bool_t  (WINRPCAPI * xp_recv)(struct SVCXPRT FAR *xprt, struct rpc_msg FAR *msg);
			/* receive incomming requests */
		enum xprt_stat (WINRPCAPI * xp_stat)(struct SVCXPRT FAR *xprt);
			/* get transport status */
		bool_t  (WINRPCAPI * xp_getargs)(
					struct SVCXPRT FAR *xprt,
					xdrproc_t xdr_args,
					caddr_t args_ptr);
			/* get arguments */
		bool_t  (WINRPCAPI * xp_reply)(struct SVCXPRT FAR *xprt, struct rpc_msg FAR *msg);
			/* send reply */
		bool_t  (WINRPCAPI * xp_freeargs)(struct SVCXPRT FAR *xprt,
					xdrproc_t xdr_args,
					caddr_t args_ptr);  /* free mem allocated for args */
		void    (WINRPCAPI *xp_destroy)(struct SVCXPRT FAR *xprt); /*destry this struct */
	} FAR *xp_ops;
	int     xp_addrlen;  /* length of remote address */
	struct sockaddr_in xp_raddr;     /* remote address */
	struct opaque_auth xp_verf;  /* raw response verifier */
	caddr_t     xp_p1;       /* private */
	caddr_t     xp_p2;       /* private */
} SVCXPRT;


#define RPCBASEERROR (0)
#define RPC_SUCCESS (0)		/* call succeeded */

#define RPC_ERROR_RESET()       {\
				WSASetLastError(0); \
				RPCSetLastError(RPC_SUCCESS);\
				}

/* common, highly portable errors
*/
	/* local errors */
#define RPC_CANTENCODEARGS (RPCBASEERROR + 1)	/* can't encode arguments */
#define RPC_CANTDECODERES (RPCBASEERROR + 2)	/* can't decode results */
#define RPC_CANTSEND (RPCBASEERROR + 3)		/* failure in sending call */
#define RPC_CANTRECV (RPCBASEERROR + 4)		/* failure in receiving result */
#define RPC_TIMEDOUT (RPCBASEERROR + 5)		/* call timed out */
	/* remote errors */
#define RPC_VERSMISMATCH (RPCBASEERROR + 6)	/* rpc versions not compatible */
#define RPC_AUTHERROR (RPCBASEERROR + 7)	/* authentication error */
#define RPC_PROGUNAVAIL (RPCBASEERROR + 8)	/* program not available */
#define RPC_PROGVERSMISMATCH (RPCBASEERROR + 9)	/* program version mismatched */
#define RPC_PROCUNAVAIL (RPCBASEERROR + 10)	/* procedure unavailable */
#define RPC_CANTDECODEARGS (RPCBASEERROR + 11)	/* decode arguments error */
#define RPC_SYSTEMERROR (RPCBASEERROR + 12)	/* generic "other problem" */
	/* Portmapper & clnt_create errors */
#define RPC_UNKNOWNHOST (RPCBASEERROR + 13)	/* unknown host name */
#define RPC_PMAPFAILURE (RPCBASEERROR + 14)	/* portmapper failed in its call */
#define RPC_PROGNOTREGISTERED (RPCBASEERROR + 15) /* remote program is not registered */
#define RPC_FAILED (RPCBASEERROR + 16)		/* unspecified error */
#define RPC_UNKNOWNPROTO (RPCBASEERROR + 17)	/* unknown protocol */

/* windows specific errors */
#define RPC_ENOTINITIALIZED     (RPCBASEERROR + 20)
#define RPC_ISBLOCKING          (RPCBASEERROR + 21)
#define RPC_ASYNCINPROGRESS     (RPCBASEERROR + 22)
#define RPC_EINVALID            (RPCBASEERROR + 23)
#define RPC_EINTR               (RPCBASEERROR + 24)
#define RPC_ADDRINUSE           (RPCBASEERROR + 25)
#define RPC_ASYNCCANCELLED      (RPCBASEERROR + 26)

/* authentication errors */
#if 0
#define RPC_AUTH_OK             (0)
#define RPC_AUTH_BADCRED        (RPCBASEERROR + 101)
#define RPC_AUTH_REJECTEDCRED   (RPCBASEERROR + 102)
#define RPC_AUTH_BADVERF        (RPCBASEERROR + 103)
#define RPC_AUTH_REJECTEDVERF   (RPCBASEERROR + 104)
#define RPC_AUTH_TOOWEAK        (RPCBASEERROR + 105)
#define RPC_AUTH_INVALIDRESP    (RPCBASEERROR + 106)
#define RPC_AUTH_FAILED         (RPCBASEERROR + 107)
#endif

/* for compatibility */
#define AUTH_OK             (0)
#define AUTH_BADCRED        (RPCBASEERROR + 101)
#define AUTH_REJECTEDCRED   (RPCBASEERROR + 102)
#define AUTH_BADVERF        (RPCBASEERROR + 103)
#define AUTH_REJECTEDVERF   (RPCBASEERROR + 104)
#define AUTH_TOOWEAK        (RPCBASEERROR + 105)
#define AUTH_INVALIDRESP    (RPCBASEERROR + 106)
#define AUTH_FAILED         (RPCBASEERROR + 107)

/* MS windows ERRORs ... */
#define RPC_WIN_BADHANDLE   (RPCBASEERROR + 200)
#define RPC_APPL_MAX        (RPCBASEERROR + 201)
#define RPC_CONNECT_CLOSE   (RPCBASEERROR + 202)

enum auth_stat {
   RPC_AUTH_OK=0,
   RPC_AUTH_BADCRED=101,
   RPC_AUTH_REJECTEDCRED=102,
   RPC_AUTH_BADVERF=103,
   RPC_AUTH_REJECTEDVERF=104,
   RPC_AUTH_TOOWEAK=105,
   RPC_AUTH_INVALIDRESP=106,
   RPC_AUTH_FAILED=107
};


/*
 * Error info.
 */
struct rpc_err {
	enum clnt_stat re_status;
	union {
		int RE_errno;       /* related system error */
			enum auth_stat RE_why;   /* why the auth error occurred */
		struct {
			u_long low; /* lowest verion supported */
			u_long high;    /* highest verion supported */
		} RE_vers;
		struct {        /* maybe meaningful if RPC_FAILED */
			long s1;
			long s2;
		} RE_lb;        /* life boot & debugging only */
	} ru;
#define re_errno    ru.RE_errno
#define re_why      ru.RE_why
#define re_vers     ru.RE_vers
#define re_lb       ru.RE_lb
};


 /* rpc message structures */

#define RPC_MSG_VERSION     ((u_long) 2)
#define RPC_SERVICE_PORT    ((u_short) 2048)

/*
 * Bottom up definition of an rpc message.
 * NOTE: call and reply use the same overall stuct but
 * different parts of unions within it.
 */

enum msg_type {
	CALL=0,
	REPLY=1
};

enum reply_stat {
	MSG_ACCEPTED=0,
	MSG_DENIED=1
};

enum accept_stat {
	SUCCESS=0,
	PROG_UNAVAIL=1,
	PROG_MISMATCH=2,
	PROC_UNAVAIL=3,
	GARBAGE_ARGS=4,
	SYSTEM_ERR=5
};

enum reject_stat {
	RPC_MISMATCH=0,
	AUTH_ERROR=1
};

/*
 * Reply part of an rpc exchange
 */

/*
 * Reply to an rpc request that was accepted by the server.
 * Note: there could be an error even though the request was
 * accepted.
 */
struct accepted_reply {
	struct opaque_auth  ar_verf;
	enum accept_stat    ar_stat;
	union {
		struct {
			u_long  low;
			u_long  high;
		} AR_versions;
		struct {
			caddr_t where;
			xdrproc_t proc;
		} AR_results;
		/* and many other null cases */
	} ru;
#define ar_results  ru.AR_results
#define ar_vers     ru.AR_versions
};

/*
 * Reply to an rpc request that was rejected by the server.
 */
struct rejected_reply {
	enum reject_stat rj_stat;
	union {
		struct {
			u_long low;
			u_long high;
		} RJ_versions;
		enum auth_stat RJ_why;  /* why authentication did not work */
	} ru;
#define rj_vers ru.RJ_versions
#define rj_why  ru.RJ_why
};

/*
 * Body of a reply to an rpc request.
 */
struct reply_body {
	enum reply_stat rp_stat;
	union {
		struct accepted_reply RP_ar;
		struct rejected_reply RP_dr;
	} ru;
#define rp_acpt ru.RP_ar
#define rp_rjct ru.RP_dr
};

/*
 * Body of an rpc request call.
 */
struct call_body {
	u_long cb_rpcvers;  /* must be equal to two */
	u_long cb_prog;
	u_long cb_vers;
	u_long cb_proc;
	struct opaque_auth cb_cred;
	struct opaque_auth cb_verf; /* protocol specific - provided by client */
};

/*
 * The rpc message
 */
struct rpc_msg {
	u_long          rm_xid;
	enum msg_type       rm_direction;
	union {
		struct call_body RM_cmb;
		struct reply_body RM_rmb;
	} ru;
#define rm_call     ru.RM_cmb
#define rm_reply    ru.RM_rmb
};
#define acpted_rply ru.RM_rmb.ru.RP_ar
#define rjcted_rply ru.RM_rmb.ru.RP_dr


/* for compatibility */
#define CLNT_CALL(hcl, proc, xdrin, in, xdrout, out, tmo) \
		clnt_call(hcl, proc, xdrin, in, xdrout, out, tmo)
#define CLNT_ABORT(hcl)                 clnt_abort(hcl)
#define CLNT_DESTROY(hcl)               clnt_destroy(hcl)
#define CLNT_GETERR(hcl, where)         clnt_geterr(hcl, where)
#define CLNT_FREERES(hcl, proc, where)  clnt_freeres(hcl, proc, where)
#define CLNT_CONTROL(hcl, what, where)  clnt_control(hcl, what, where)    

/*
 * control operations that apply to all transports
 */
#define	CLSET_TIMEOUT		1	/* set timeout (timeval) */
#define	CLGET_TIMEOUT		2	/* get timeout (timeval) */
#define	CLGET_SERVER_ADDR	3	/* get server's address (sockaddr) */
#define	CLGET_FD		6	/* get connections file descriptor */
#ifdef WINRPC_OBSOLETE_CLAUTH
/*
** CLSET_AUTH and CLGET_AUTH as defined by the ill-fated
** RPC4WIN specification conflicts with both ONC RPC and TIRPC.
** The WINRPC programmer should instead use clnt->cl_auth.
*/
#define CLGET_AUTH          6   /* get the client's current auth structure */
#define CLSET_AUTH          7   /* change to a new auth structure */
#endif /* WINRPC_OBSOLETE_CLAUTH */
#ifdef WINRPC_FUTURE
/*
** These may be supported in a future release.
*/
#define	CLGET_SVC_ADDR		7	/* get server's address (netbuf) */
#define	CLSET_FD_CLOSE		8	/* close fd while clnt_destroy */
#define	CLSET_FD_NCLOSE		9	/* Do not close fd while clnt_destroy */
#define	CLGET_XID 		10	/* Get xid */
#define	CLSET_XID		11	/* Set xid */
#define	CLGET_VERS		12	/* Get version number */
#define	CLSET_VERS		13	/* Set version number */
#define	CLGET_PROG		14	/* Get program number */
#define	CLSET_PROG		15	/* Set program number */
#define	CLSET_SVC_ADDR		16	/* get server's address (netbuf) */
#define	CLSET_PUSH_TIMOD	17	/* push timod if not already present */
#define	CLSET_POP_TIMOD		18	/* pop timod */
#endif /* WINRPC_FUTURE */
/*
 * Connectionless only control operations
 */
#define	CLSET_RETRY_TIMEOUT 4   /* set retry timeout (timeval) */
#define	CLGET_RETRY_TIMEOUT 5   /* get retry timeout (timeval) */



/*
 * RPCTEST is a test program which is accessable on every rpc
 * transport/port.  It is used for testing, performance evaluation,
 * and network administration.
 */

#define RPCTEST_PROGRAM     ((u_long)1)
#define RPCTEST_VERSION     ((u_long)1)
#define RPCTEST_NULL_PROC   ((u_long)2)
#define RPCTEST_NULL_BATCH_PROC ((u_long)3)

/* By convention, procedure 0 takes null arguments and returns them */
#define NULLPROC ((u_long)0)

#define UDPMSGSIZE  8800    /* rpc imposed limit on udp msg size */
#define RPCSMALLMSGSIZE 400 /* a more reasonable packet size */


#if defined(VSL_IPXSTACK)
#define IPXBPORT        ((u_short)36889)
#define SPXBPORT        ((u_short)36890) 
/*??? #define RPCBVERS        ((u_long)3) */
#endif

#define PMAPPORT        ((u_short)111)
#define PMAPVERS        ((u_long)2)

struct pmap {
	long unsigned pm_prog;
	long unsigned pm_vers;
	long unsigned pm_prot;
	long unsigned pm_port;
};

struct pmaplist {
	struct pmap pml_map;
	struct pmaplist FAR *pml_next;
};
 
typedef struct pmap PMAP;
typedef struct pmaplist PMAPLIST;       

#define PMAPPROG        ((u_long)100000)
#define PMAPVERS_PROTO      ((u_long)2)
#define PMAPVERS_ORIG       ((u_long)1)
#define PMAPPROC_NULL       ((u_long)0)
#define PMAPPROC_SET        ((u_long)1)
#define PMAPPROC_UNSET      ((u_long)2)
#define PMAPPROC_GETPORT    ((u_long)3)
#define PMAPPROC_DUMP       ((u_long)4)
#define PMAPPROC_CALLIT     ((u_long)5)


struct rmtcallargs {
	u_long prog, vers, proc, arglen;
	caddr_t args_ptr;
	xdrproc_t xdr_args;
};
struct rmtcallres {
	u_long FAR *port_ptr;
	u_long resultslen;
	caddr_t results_ptr;
	xdrproc_t xdr_results;
};


/*
 * Server side transport handle
 */


/*
 * Operations defined on an SVCXPRT handle
 *
 * SVCXPRT      *xprt;
 * struct rpc_msg   *msg;
 * xdrproc_t         xargs;
 * caddr_t       argsp;
 */


/* for compatibility, allow all caps calls */
#define SVC_RECV(xprt, msg)             svc_recv(xprt, msg)
#define SVC_STAT(xprt)                  svc_stat(xprt)
#define SVC_GETARGS(xprt, proc, where)  svc_getargx(xprt, proc, where)
#define SVC_REPLY(xprt, msg)            svc_reply(xprt, msg)
#define SVC_FREEARGS(xprt, proc, where) svc_freeargs(xprt, proc, where)
#define SVC_DESTROY(xprt)               svc_destroy(xprt)
#define svc_getcaller(x) (&(x)->xp_raddr)

/*
 * Service request
 */
struct svc_req {
	u_long      rq_prog;    /* service program number */
	u_long      rq_vers;    /* service protocol version */
	u_long      rq_proc;    /* the desired procedure */
	struct opaque_auth rq_cred; /* raw creds from the wire */
	caddr_t     rq_clntcred;    /* read only cooked cred */
	SVCXPRT FAR *   rq_xprt;    /* associated transport */
};


typedef bool_t (WINRPCAPI *resultproc_t)(caddr_t, struct sockaddr_in FAR *);

/* Threaded Material */
enum svc_mt_e {
     SVC_NONE=0,
     SVC_WAKEUP=1,
     SVC_DONE=2
};

enum serverrun_e {
     SVC_RUNNING=0,
     SVC_NOTRUNNING=1
};


/* RPC XDR Functions */

#ifdef __cplusplus
extern "C" {
#endif


/* 
 * If a creation fails, the following allows the user to figure out why.
 */
#define rpc_createerr (*(get_rpc_createerr()))
typedef struct _rpc_createerr { /* avoid conflict with "rpc_createerr" macro */
	enum clnt_stat cf_stat;
	struct rpc_err cf_error; /* useful when cf_stat == RPC_PMAPFAILURE */
} _rpc_createerr;

extern _rpc_createerr FAR * WINRPCAPI get_rpc_createerr(void);

bool_t WINRPCAPI xdr_array(XDR FAR *xdrs, caddr_t FAR *addrp,
	u_int FAR *sizep,
	u_int maxsize, u_int elsize, xdrproc_t elproc);
								
bool_t WINRPCAPI xdr_authunix_parms(XDR FAR *xdrs,
										struct authunix_parms FAR *p);
bool_t WINRPCAPI xdr_bool(XDR FAR *xdrs, bool_t FAR *bp);
bool_t WINRPCAPI xdr_bytes(XDR FAR *xdrs, char FAR * FAR *cpp,
	u_int FAR *sizep, u_int maxsize);
bool_t WINRPCAPI xdr_ez_binary(XDR FAR *xdrs, char FAR * FAR *cpp,
	u_long FAR *sizep, u_long maxsize);
bool_t WINRPCAPI xdr_char(XDR FAR *xdrs, char FAR *cp);
bool_t WINRPCAPI xdr_double(XDR FAR *xdrs, double FAR *dp);
bool_t WINRPCAPI xdr_enum(XDR FAR *xdrs, enum_t FAR *ep);
bool_t WINRPCAPI xdr_float(XDR FAR *xdrs, float FAR *fp);
void   WINRPCAPI xdr_free(xdrproc_t proc, char FAR *objp);
bool_t WINRPCAPI xdr_int(XDR FAR *xdrs, int FAR *ip);
bool_t WINRPCAPI xdr_long(XDR FAR *xdrs, long FAR *lp);
bool_t WINRPCAPI xdr_netobj(XDR FAR *xdrs, struct netobj FAR *np);
bool_t WINRPCAPI xdr_opaque(XDR FAR *xdrs, caddr_t cp, u_int cnt);
bool_t WINRPCAPI xdr_opaque_auth(XDR FAR *xdrs, struct opaque_auth FAR *ap);
bool_t WINRPCAPI xdr_pmap(XDR FAR *xdrs, struct pmap FAR *regs);
bool_t WINRPCAPI xdr_pmaplist(XDR FAR *xdrs,
		struct pmaplist FAR * FAR *rp);
bool_t WINRPCAPI xdr_pointer(XDR FAR *xdrs, char FAR * FAR *objpp,
		u_int obj_size, xdrproc_t xdr_obj);
bool_t WINRPCAPI xdr_reference(XDR FAR *xdrs, caddr_t FAR  *pp,
		u_int size, xdrproc_t proc);
bool_t WINRPCAPI xdr_rmtcall_args(XDR FAR *xdrs,
		struct rmtcallargs FAR *cap);
bool_t WINRPCAPI xdr_rmtcallres(XDR FAR *xdrs,
		struct rmtcallres FAR *crp);
bool_t WINRPCAPI xdr_short(XDR FAR *xdrs, short FAR *sp);
bool_t WINRPCAPI xdr_string(XDR FAR *xdrs, char FAR * FAR *cpp,
		u_int maxsize);  

bool_t WINRPCAPI xdr_u_char(XDR FAR *xdrs, u_char FAR *cp);
bool_t WINRPCAPI xdr_u_int(XDR FAR *xdrs, u_int FAR *up);
bool_t WINRPCAPI xdr_u_long(XDR FAR *xdrs, u_long FAR *ulp);
bool_t WINRPCAPI xdr_u_short(XDR FAR *xdrs, u_short FAR *usp);
bool_t WINRPCAPI xdr_union(XDR FAR *xdrs, enum_t FAR *dscmp,
			   char FAR *unp, struct xdr_discrim FAR *choices, xdrproc_t dfault);
bool_t WINRPCAPI xdr_vector(XDR FAR *xdrs, char FAR *basep, u_int nelem,
									u_int elemsize, xdrproc_t xdr_elem);
bool_t WINRPCAPI xdr_void(XDR FAR *, void FAR *);
bool_t WINRPCAPI xdr_wrapstring(XDR FAR *xdrs, char FAR * FAR *cpp);
#ifdef USEXDRSTDIO
void WINRPCAPI xdrstdio_create( XDR FAR *xdrs, FILE FAR *file, enum xdr_op op);
FILE * WINRPCAPI xdrstdio_fopen(char *filename, char *type);
int WINRPCAPI xdrstdio_fclose(FILE *fd);
#endif
void WINRPCAPI xdrmem_create( XDR FAR *xdrs, caddr_t addr, u_int size, enum xdr_op op);
bool_t WINRPCAPI xdrrec_skiprecord( XDR FAR *xdrs );
bool_t WINRPCAPI xdrrec_eof( XDR FAR *xdrs );
bool_t WINRPCAPI xdrrec_endofrecord( XDR FAR *xdrs, bool_t sendnow );


typedef int (FAR xdrrec_iofunc)( void FAR * handle, caddr_t buf, int len );

void WINRPCAPI xdrrec_create( XDR FAR * xdrs, u_int sendsize, u_int recvsize, caddr_t tcp_handle, xdrrec_iofunc * readit, xdrrec_iofunc * writeit);


/* EZ-RPC XDR Functions for freeing complex arguments. EZRPCxxx.DLL */
bool_t WINRPCAPI xdr_ez_free( 
	char FAR *p, xdrproc_t elproc, u_int opt 
	);
bool_t WINRPCAPI xdr_ez_free_pointer( 
	char FAR * FAR * pp, u_int elsize, xdrproc_t elproc, u_int opt 
	);
bool_t WINRPCAPI xdr_ez_free_array( 
	caddr_t FAR * pp, u_int nelem, u_int elsize, xdrproc_t elproc, u_int opt
	);
bool_t WINRPCAPI xdr_ez_free_parray( 
	caddr_t FAR * FAR * ppp, u_int nelem, u_int elsize, xdrproc_t elproc, u_int opt
	);
bool_t WINRPCAPI xdr_ez_free_p2array( 
	caddr_t FAR * FAR * ppp, u_int nelem, u_int elsize, xdrproc_t elproc, u_int optional 
	);
bool_t WINRPCAPI xdr_ez_free_ppointer( 
	char FAR * FAR * FAR * ppp, u_int elsize, xdrproc_t elproc, u_int optional 
	);
bool_t WINRPCAPI xdr_ez_free_vector( 
	char FAR *p, u_int nelem, u_int elsize, xdrproc_t elproc, u_int optional 
	);
bool_t WINRPCAPI xdr_ez_free_pvector( 
	char FAR *FAR *pp, u_int nelem, u_int elsize, xdrproc_t elproc, u_int optional 
	);
 

 /* RPC Authentication functions */
AUTH FAR * WINRPCAPI authnone_create(void);
AUTH FAR * WINRPCAPI authunix_create(   char FAR *machname,
					u_long uid,
					u_long gid,
					int len,
					gid_t FAR *aup_gids);
AUTH FAR * WINRPCAPI authunix_create_default(void);

/* RPC Client Functions */

char FAR * WINRPCAPI clnt_spcreateerror(char FAR *msg);

int WINRPCAPI callrpc(char FAR *host, u_long prognum, u_long versnum,
		u_long procnum, xdrproc_t inproc, char FAR *in, xdrproc_t outproc,
		char FAR *out);
		
enum clnt_stat WINRPCAPI clnt_broadcast(u_long prog, u_long vers, u_long proc, 
	xdrproc_t xargs, caddr_t argsp, xdrproc_t xresults, caddr_t resultsp, 
	resultproc_t eachresult);
	
enum clnt_stat WINRPCAPI clnt_call(CLIENT FAR *, u_long, xdrproc_t, caddr_t,
		xdrproc_t, caddr_t, struct timeval);

int WINRPCAPI clnt_control(CLIENT FAR *, u_int, caddr_t);

CLIENT FAR * WINRPCAPI clnt_create(
	char FAR *hostname,
	u_long prog,
	u_long vers,
	char FAR *proto);

CLIENT FAR * WINRPCAPI clnt_ez_create(
	char FAR *hostname,
	u_long prog,
	u_long vers,
	char FAR *proto,
	u_int sendsz,
	u_int recvsz);

void WINRPCAPI clnt_destroy(CLIENT FAR *);

int WINRPCAPI clnt_freeres(CLIENT FAR *, xdrproc_t, caddr_t);
		
void WINRPCAPI clnt_geterr(CLIENT FAR *, struct rpc_err FAR *);

void WINRPCAPI clnt_perror(CLIENT FAR *, char FAR *);
void WINRPCAPI clnt_perrno(enum clnt_stat);
void WINRPCAPI clnt_pcreateerror(char FAR *);
char FAR * WINRPCAPI clnt_sperror(CLIENT FAR *, char FAR *);
char FAR * WINRPCAPI clnt_sperrno(enum clnt_stat);

CLIENT FAR * WINRPCAPI clnttcp_create(
	struct sockaddr_in FAR *raddr,
	u_long prog,
	u_long vers,
	SOCKET FAR *portp,
	u_int sendsz,
	u_int recvsz);

CLIENT FAR * WINRPCAPI clntudp_bufcreate(
	struct sockaddr_in FAR *raddr,
	u_long program,
	u_long version,
	struct timeval wait,
	SOCKET FAR  *portp,
	u_int sendsz,
	u_int recvsz);

 CLIENT FAR * WINRPCAPI clntudp_create(
	struct sockaddr_in FAR *raddr,
	u_long program,
	u_long version,
	struct timeval wait,
	SOCKET FAR *portp);

/* RPC Server Functions */
enum svc_mt_e WINRPCAPI svc_mt_check_interrupt(fd_set FAR *); 
int           WINRPCAPI svc_mt_interrupt_create(void);
void          WINRPCAPI svc_mt_interrupt(enum svc_mt_e);
SOCKET        WINRPCAPI svc_mt_getsocket(void);
enum serverrun_e WINRPCAPI svc_mt_check_status(void);
int           WINRPCAPI svc_mt_set_status(enum serverrun_e status);
HANDLE   WINRPCAPI svc_mt_server_run (void(FAR *runloop) (void));


void          WINRPCAPI svc_fd_set(SOCKET socket, fd_set FAR *);
void          WINRPCAPI svc_fd_clr(SOCKET socket, fd_set FAR *);
void          WINRPCAPI svc_fd_zero(fd_set FAR *);
int           WINRPCAPI svc_fd_isset(SOCKET socket, fd_set FAR * );
int           WINRPCAPI svc_fd_count(fd_set FAR *);
SOCKET        WINRPCAPI svc_fd_getsocket(fd_set FAR *, int);

typedef enum auth_stat (WINRPCAPI * svc_auth_handler)( struct svc_req FAR *,
struct rpc_msg FAR * );
int WINRPCAPI svc_auth_reg(int cred_flavor, svc_auth_handler handler);


void WINRPCAPI svc_destroy(SVCXPRT FAR *);
bool_t WINRPCAPI svc_freeargs(SVCXPRT FAR *, xdrproc_t, caddr_t);
bool_t WINRPCAPI svc_getargs(SVCXPRT FAR *, xdrproc_t, caddr_t);
/* obsolete: fd_set WINRPCAPI svc_getfds(void); */
fd_set FAR * WINRPCAPI svc_getfdset(fd_set FAR *);
void WINRPCAPI svc_getreq(int sock);
void WINRPCAPI svc_getreqset(fd_set FAR *);
bool_t WINRPCAPI svc_register(SVCXPRT FAR * xprt, u_long prog,
		u_long vers,
		void (WINRPCAPI *dispatch)(struct svc_req FAR *, SVCXPRT FAR *),
		int protocol);
void WINRPCAPI svc_run(void);
bool_t WINRPCAPI svc_sendreply(SVCXPRT FAR * xprt, xdrproc_t xdr_results,
									   caddr_t xdr_location);
enum xprt_stat WINRPCAPI svc_stat(SVCXPRT FAR *);
void WINRPCAPI svc_unregister(u_long prog, u_long vers);
bool_t WINRPCAPI svc_recv(SVCXPRT FAR * xprt, struct rpc_msg FAR * msg);
bool_t WINRPCAPI svc_reply(SVCXPRT FAR * xprt, struct rpc_msg FAR * msg);
void WINRPCAPI svcerr_auth(SVCXPRT FAR * xprt, enum auth_stat why);
void WINRPCAPI svcerr_noproc(SVCXPRT FAR * xprt);
void WINRPCAPI svcerr_noprog(SVCXPRT FAR * xprt);
void WINRPCAPI svcerr_progvers(SVCXPRT FAR * xprt, u_long low_vers,
				   u_long high_vers);
void WINRPCAPI svcerr_decode(SVCXPRT FAR * xprt);
void WINRPCAPI svcerr_systemerr(SVCXPRT FAR * xprt);
void WINRPCAPI svcerr_weakauth(SVCXPRT FAR * xprt);

SVCXPRT FAR * WINRPCAPI svctcp_create(SOCKET port, u_int sendsize,
										  u_int recvsize);

SVCXPRT FAR * WINRPCAPI svcfd_create(int fd, u_int sendsize, 
				     u_int recvsize);
SVCXPRT FAR * WINRPCAPI svcudp_bufcreate(SOCKET port, u_int sendsz,
												 u_int recvsz); 
SVCXPRT FAR * WINRPCAPI svcudp_create(SOCKET port);

/* portmapper functions */

u_short WINRPCAPI pmap_getport(struct sockaddr_in FAR *address,
						u_long program, u_long version, u_int protocol);

struct pmaplist FAR * WINRPCAPI pmap_getmaps
						(struct sockaddr_in FAR *address);

enum clnt_stat WINRPCAPI pmap_rmtcall(struct sockaddr_in FAR *addr,
			u_long prog, u_long vers, u_long proc, xdrproc_t xdrargs,
			caddr_t argsp, xdrproc_t xdrres, caddr_t resp, struct timeval tout,
			u_long FAR *port_ptr);
			


bool_t WINRPCAPI pmap_set(u_long program, u_long version,
								  int protocol, u_short port);
								  
bool_t WINRPCAPI pmap_unset(u_long program, u_long version);


/*
 * Windows specific asynchronous calls
 *
 */

int     WINRPCAPI RPCInit(HWND);
int     WINRPCAPI RPCExit(void);
int     WINRPCAPI RPCControl(int cmd, long param);	/* see options below */

/* RPCControl commands:
*/
#define RPC_MEM_SHARE		1	/* allocate sharable memory */
#define RPC_MEM_NOSHARE		2	/* allocate task-private memory */
#define RPC_MEM_SUBALLOCATE	3	/* sub-allocate to save selectors */
#define RPC_MEM_NOSUBALLOCATE	4	/* use GlobalAlloc for each alloc */
#define RPC_SET_XDRREC_BUFSIZE	10	/* set stream record size (def=4000) */
#define RPC_POLL_ALL_SOCKETS	11	/* workaround for flaky TCP/IP stacks */
#define RPC_GET_PROTOCOL		12	/* get proto in use: RPC_INIT_{TCP, SPX} */

#define RPC_SVC_MT_INIT       20      /* Turn on Multi-threaded server model*/
#define RPC_SVC_MT_GET        21      /* Is Multi-threaded server model on ?*/ 




#define RPC_DISABLE_ASYNC_SVC	22	/* disable AsyncSelect for services */
#define RPC_ENABLE_ASYNC_SVC	23	/* enable AsyncSelect for services */
/* old:	RPC_SVC_MAX_XPRTS       24      ** This function is obsolete */

#define RPC_CLNT_TIMEOUT_CONNECT	24   /* time to wait for connection to portmapper*/

#define RPC_GET_FD_SETSIZE		25      /* return current FD_SETSIZE value */
#define RPC_SET_FD_SETSIZE		26      /* set new FD_SETSIZE value */

#define RPC_DISABLE_ERROR_BOX	0x80	/* disable error message boxes */
#define RPC_ENABLE_ERROR_BOX	0x100	/* enable error message boxes */

FARPROC WINRPCAPI RPCSetBlockingHook(FARPROC lpNewHook); 
int     WINRPCAPI RPCUnsetBlockingHook(void);
int     WINRPCAPI RPCDefaultHook(void); /* Peek(REMOVE), Dispatch() */
int     WINRPCAPI RPCYieldHook(void); /* Peek(NOREMOVE) */
int     WINRPCAPI RPCNilHook(void); /* noop() */
int     WINRPCAPI RPCDropHook(void); /* drop mouse, keyboard, allow <alttab> */

int     WINRPCAPI RPCGetLastError(void);
void    WINRPCAPI RPCSetLastError(int error);
int     WINRPCAPI RPCCancelBlockingCall(void);
int     WINRPCAPI RPCIsBlocking(void);
int     WINRPCAPI getdtablesize(void);

/* RPCControl options:
*/
#define RPC_INIT_AUTO	100	/* automatic stack select operation (default) */
#define RPC_INIT_TCP	101	/* select TCP/IP operation */
#define RPC_INIT_SPX	102	/* select SPX/IPX operation */
#define RPC_INIT_BY_NAME 103	/* select TCP/IP, with renamed WINSOCK.DLL */

#define RPCInitTcp( hWnd ) RPCControl( RPC_INIT_TCP, (long) hWnd )
#define RPCInitSpx( hWnd ) RPCControl( RPC_INIT_SPX, (long) hWnd )
#define RPCInitByName(name) RPCControl(RPC_INIT_BY_NAME,(long)(char FAR *)name)

#define RPCInitQuiet( hWnd ) \
	RPCControl( RPC_INIT_AUTO | RPC_DISABLE_ERROR_BOX, (long) hWnd )
#define RPCInitTcpQuiet( hWnd ) \
	RPCControl( RPC_INIT_TCP | RPC_DISABLE_ERROR_BOX, (long) hWnd )
#define RPCInitSpxQuiet( hWnd ) \
	RPCControl( RPC_INIT_SPX | RPC_DISABLE_ERROR_BOX, (long) hWnd )

/*
**  Wrappers of WinSock functions for peer-to-peer RPC.
*/
int PASCAL FAR ez_connect (SOCKET s, struct sockaddr FAR * name, int namelen);
int PASCAL FAR ez_recv (SOCKET s, char FAR * buf, int len, int flags);
int PASCAL FAR ez_recvfrom (SOCKET s, char FAR * buf, int len, int flags,
						 struct sockaddr FAR *from, int FAR * fromlen);
int PASCAL FAR ez_select (int nfds, fd_set FAR *readfds, fd_set FAR *writefds,
					   fd_set FAR *exceptfds, struct timeval FAR *timeout);
int PASCAL FAR ez_send (SOCKET s, char FAR * buf, int len, int flags);
int PASCAL FAR ez_sendto (SOCKET s, char FAR * buf, int len, int flags,
					   struct sockaddr FAR *to, int tolen);
int PASCAL FAR ez___WSAFDIsSet(SOCKET, fd_set FAR *);
int PASCAL FAR ez_WSAGetLastError(void);

struct protoent FAR * PASCAL FAR ez_getprotobyname(const__ char FAR *name);

int PASCAL FAR ez_gethostname (char FAR * name, int namelen);
struct hostent FAR * PASCAL FAR ez_gethostbyname(const__ char FAR * name);

/*
** SIZEOF_FD_SET is a macro to return the size of a
** dynamically-sized "fd_set" structure containing "count" fds.
**
** FD_COPY is a macro to copy "fd_set"s, since the user can
** change FD_SETSIZE from its default of 64.  Note that this 
** presumes the Windows Sockets version 1.1 definition of "fd_set", 
** as shown in WINSOCK.H.
**
** We recommended against defining FD_SETSIZE to less than 64,
** to avoid certain Win16 WINSOCK problems.
**
** The user must 
**
**	1)	insure that the number of services registered is
**		less than FD_SETSIZE - 1.
**	2)	consistently #define it before including winsock.h or winrpc.h,
**	3)	insure that fd_count does not exceed FD_SETSIZE, and
**	4)	call RPCControl(RPC_SET_FD_SETSIZE, size ) to tell WinRPC of
**		the non-standard setting.
**	5)	#define FD_SETSIZE large enough to handle as many connections
**		as may come in.
**
** In modules that don't have access to FD_SETSIZE, but must
** still manipulate "fd_set"s, these macros should help.
**
** This size is the larger of an fd_set[64], or an fd_set[count].
*/
#define SIZEOF_FD_SET(count) \
	( \
		max(sizeof(struct fd_set),\
		    sizeof(struct fd_set) + ((count) - FD_SETSIZE) * sizeof(SOCKET)) \
	)
#ifndef FD_COPY     /* someday winsock.h should define this */
#define FD_COPY(dst, src) \
do { \
	unsigned int i; \
\
	for (i = 0; i < (src)->fd_count; i++) { \
		(dst)->fd_array[i] = (src)->fd_array[i]; \
	} \
	(dst)->fd_count = i; \
} while(0)
#endif /* FD_COPY */

/*
**  Wrappers of DLL memory allocation functions,
**  for access from XDR layers.
*/
void FAR *  WINRPCAPI ez_calloc(unsigned int nitems, unsigned int size);
void FAR *  WINRPCAPI ez_malloc(unsigned int size);
void FAR *  WINRPCAPI ez_realloc(void FAR *block, unsigned int size);

void        WINRPCAPI ez_free(void FAR *block);

void FAR *  WINRPCAPI ez_farcalloc(unsigned long nunits, unsigned long unitsz);
void        WINRPCAPI ez_farfree(void FAR *block);
void FAR *  WINRPCAPI ez_farmalloc(unsigned long nbytes);
void FAR *  WINRPCAPI ez_farrealloc(void FAR *oldblock, unsigned long nbytes);

void WINRPCAPI ez_errorMsg(char FAR *msgTitle, char FAR *format);
u_long WINRPCAPI ez_getrandomlong( void );

#ifdef __cplusplus
}
#endif

#endif /* _WINRPC_H_ */
